# -*- coding: utf-8 -*-
import csv, os
import logging
from django.db import transaction
from django.core.management.base import BaseCommand
from django.conf import settings

from genetics.models import HgncGene, OmimGene, OmimTitle, OmimPhenotype

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = '''Imports Gene data from HGNC and OMIM'''

    def handle(self, *args, **options):
        ############# HGNC GENE
        # HGNC ID	Status	Approved symbol	Approved name	Chromosome location	Orphanet ID	OMIM ID
        # 0 HGNC ID   1 Approved symbol     2 Approved name      3 Status      4 Alias symbols     5 Chromosome       6 RefSeq IDs    7 Alias names     8 OMIM ID(supplied by OMIM)
        # ==> genetics.models.HgncGene
        # hgnc_id	Status	approved_symbol	approved_name	chromosome_location	orphanet_id	omim_id
        hgnc_gene_columns = {
            'hgnc_id': 0,
            'approved_symbol': 1,
            'approved_name': 2,
            'alias_symbols': 4,
            'alias_names': 2,
            'chromosome_location': 5,
            'omim_id': 8
        }
        ############# OMIM MORBID MAP
        # Phenotype	Gene Symbols	MIM Number	Cyto Location
        # ==> genetics.models.OmimGene
        # ---------	gene_symbols	omim_id 	chromosome_location
        # ==> genetics.models.OmimPhenotype
        # phenotype
        morbid_map_columns = {
            'phenotype': 0,
            'gene_symbols': 1,
            'mim_number': 2,
            'chromosome_location': 3,
        }
        ############# OMIM MIMTITLES
        # Prefix	MIM Number	Preferred Title; symbol	Alternative Title(s); symbol(s)
        # ==> genetics.models.OmimTitle
        # prefix	omim_gene	preferred               alternative
        mim_titles_columns = {
            'prefix': 0,
            'mim_number': 1,
            'preferred': 2,
            'alternative': 3,
        }

        try:
            with transaction.atomic():
                ############# HGNC GENE
                if True:
                    this_dir = os.path.dirname(os.path.abspath(__file__))
                    file_path = ("%s/../../data/hgnc/HGNC Gene w alias.csv" % this_dir)
                    logger.info("Importing HGNC GENE w alias.csv")
                    with open(file_path, 'r', errors='ignore') as file_hgnc_gene:
                        csv_reader = csv.reader(file_hgnc_gene, delimiter=',', quotechar='"')
                        for row in csv_reader:
                            if row[0] != "HGNC ID": #salto la prima riga
                                try:
                                    # se non esiste già (ho verificato: hgnc_id è univoco)
                                    if not HgncGene.objects.filter(hgnc_id=row[hgnc_gene_columns['hgnc_id']]).exists():
                                        HgncGene.objects.create(
                                            hgnc_id=row[hgnc_gene_columns['hgnc_id']],
                                            approved_symbol=row[hgnc_gene_columns['approved_symbol']],
                                            approved_name=row[hgnc_gene_columns['approved_name']],
                                            alias_symbols=row[hgnc_gene_columns['alias_symbols']],
                                            alias_names=row[hgnc_gene_columns['alias_names']],
                                            chromosome_location=row[hgnc_gene_columns['chromosome_location']],
                                            mim_number=row[hgnc_gene_columns['omim_id']],
                                        )

                                except Exception as ex:
                                    logger.error("Non blocking error importing HGNC Gene: %s" % (str(ex)))
                ############# OMIM MORBID MAP
                if True:
                    file_path = ("%s/../genetics/data/omim/morbidmap.csv" % settings.BASE_DIR)
                    logger.info("Importing OMIM morbidmap.csv")
                    with open(file_path, 'r', errors='ignore') as file_omim_morbidmap:
                        csv_reader = csv.reader(file_omim_morbidmap, delimiter=',', quotechar='"')
                        for row in csv_reader:
                            if row[0] != '# Phenotype': #salto la prima riga
                                try:
                                    # se non esiste già (ho verificato: hgnc_id è univoco)
                                    if not OmimGene.objects.filter(mim_number=row[morbid_map_columns['mim_number']]).exists():
                                        og = OmimGene.objects.create(
                                            mim_number=row[morbid_map_columns['mim_number']],
                                            gene_symbols=row[morbid_map_columns['gene_symbols']],
                                            chromosome_location=row[morbid_map_columns['chromosome_location']]
                                        )
                                    else:
                                        og = OmimGene.objects.get(mim_number=row[morbid_map_columns['mim_number']])
                                        # per ora non verifico gli altri campi ....
                                    if not og.phenotypes.filter(phenotype=row[morbid_map_columns['phenotype']]).exists():
                                        OmimPhenotype.objects.create(omim_gene=og,
                                                                     phenotype=row[morbid_map_columns['phenotype']])
                                except Exception as ex:
                                    logger.error("Non blocking error importing OMIM morbidmap: %s" % (str(ex)))
                ############# OMIM MIMTITLES
                if True:
                    file_path = ("%s/../genetics/data/omim/mimTitles.csv" % settings.BASE_DIR)
                    logger.info("Importing OMIM mimTitles.csv")
                    with open(file_path, 'r', errors='ignore') as file_omim_titles:
                        csv_reader = csv.reader(file_omim_titles, delimiter=',', quotechar='"')
                        for row in csv_reader:
                            if row[0] != 'Prefix': #salto la prima riga
                                try:
                                    # se non esiste già [ho controllato solo una riga per ciascun mim_numer]
                                    if not OmimTitle.objects.filter(omim_gene__mim_number=row[mim_titles_columns['mim_number']]).exists():
                                        og = OmimTitle.objects.create(
                                            omim_gene=OmimGene.objects.get(mim_number=row[mim_titles_columns['mim_number']]),
                                            prefix=row[mim_titles_columns['prefix']],
                                            preferred=row[mim_titles_columns['preferred']],
                                            alternative=row[mim_titles_columns['alternative']],
                                        )
                                    else:
                                        og = OmimTitle.objects.get(omim_gene__mim_number=row[mim_titles_columns['mim_number']])
                                        # per ora non verifico gli altri campi ....
                                        og.prefix = row[mim_titles_columns['prefix']]
                                        og.preferred = row[mim_titles_columns['preferred']]
                                        og.alternative = row[mim_titles_columns['alternative']]
                                        og.save()
                                except Exception as ex:
                                    # se OmimGene non esiste è perché non è presente in morbidmap
                                    logger.error("Non blocking error importing OMIM morbidmap: %s" % (str(ex)))
                                    logger.error("Row: %s" % row)
        except Exception as ex:
            logger.error("Blocking error importing HGNC / OMIM data: %s" % str(ex))
        print("END!")
        logger.info("END END END importing OMIM morbidmap END END END")



import json

from django.db.models import Q
from django.http import HttpResponse, JsonResponse
from genetics.utils import ApiInvoker
from genetics.models import OmimGene
from genetics.models import HgncGene


def gene(request):
    data = 'fail'
    try:
        if request.is_ajax():
            q = request.GET.get('term', '')
            # limito a 25 il n. di record per evitare eccessivi rallentamenti per ricerca
            genes = list(HgncGene.objects.filter(Q(approved_symbol__icontains=q) | Q(alias_symbols__icontains=q) |
                                                 Q(approved_name__icontains=q) | Q(alias_names__icontains=q) )[:25])
            # per lo stesso motivo effettuo la ricerca solo se ci sono almeno 4 caratteri
            mims = []
            if len(q) >= 4:
                mims = [og.mim_number for og in OmimGene.objects.filter(phenotypes__phenotype__icontains=q)[:25]]
            other_genes = []
            if len(mims) > 0:
                other_genes = list(HgncGene.objects.filter(mim_number__in=mims))
            all_genes = list(set(genes + other_genes))
            results = []
            for g in all_genes:
                phenotypes = " || ".join(g.omim_phenotypes)
                symbol = g.approved_symbol
                if g.alias_symbols:
                    symbol += " (%s)" % g.alias_symbols
                genes_json = {
                    'id': g.id,
                    'label': "%s - %s" % (symbol, g.approved_name),
                    'approved_symbol': g.approved_symbol,
                    'chromosome_location': g.chromosome_location,
                    'phenotypes': phenotypes,
                    'titles': g.html_omim_titles,
                    'value': "%s - %s" % (g.approved_symbol, g.approved_name),
                    'mim_number': g.mim_number
                }
                if g.mim_number:
                    pass
                    # I seek extra info on OMIM
                results.append(genes_json)
            data = json.dumps(results)
    except:
        pass
    mimetype = 'application/json'
    return HttpResponse(data, mimetype)


def transcripts(request):
    data = {
        'transcripts': [],
        'error': "",
        'status': ApiInvoker.FAILURE
    }
    try:
        if request.is_ajax():
            mutalyzer_cs_endpoint = "https://mutalyzer.nl/json/getTranscriptsByGeneName"
            url = mutalyzer_cs_endpoint + ("?build=hg38&name=%s" % request.GET['name'])
            aics = ApiInvoker(url=url)
            # https://mutalyzer.nl/json/getTranscriptsByGeneName?build=hg19&name=MAP7D2
            """
                [
                    "NM_001168465.1",
                    "NM_001168466.1",
                    "NM_001168467.1",
                    "NM_152780.2",
                    "NM_152780.3",
                    "XM_005274478.1",
                    "XM_005274479.1",
                    "XM_005274480.1",
                    "XM_005274481.1"
                ]
            """
            data['transcripts'] = aics.decoded
            if aics.exception:
                data['exception'] = aics.exception
            data['status'] = ApiInvoker.SUCCESS
        else:
            data['error'] = "Generic error."
    except:
        pass
    data['name'] = request.GET['name']
    return JsonResponse(data)


def sequence_variant(request):
    data = {
        'notes': "",
        'error': "",
        'status': ApiInvoker.FAILURE
    }
    try:
        if request.is_ajax():
            mutalyzer_cs_endpoint = "https://mutalyzer.nl/json/checkSyntax"
            url = mutalyzer_cs_endpoint + ("?variant=%s" % request.GET['sv'])
            aics = ApiInvoker(url=url)
            # https://mutalyzer.nl/json/checkSyntax?variant=AB026906.1:c.274de
            """
                {
                  "valid": false,
                  "messages": [
                    {
                      "errorcode": "EPARSE",
                      "message": "Expected \">\" (at char 17), (line:1, col:18)"
                    }
                  ]
                }
            """
            if aics.decoded["valid"]:
                data['notes'] = 'Valid syntax. '
                # se è valido sintatticamente faccio una verifica semantica
                mutalyzer_ml_endpoint = "https://mutalyzer.nl/json/runMutalyzerLight"
                url = mutalyzer_ml_endpoint + ("?variant=%s" % request.GET['sv'])
                aiml = ApiInvoker(url=url)
                if aiml.decoded['errors'] == 0:
                    try:
                        data['proteinDescriptions'] = [pd[1+pd.find(':'):] for pd in aiml.decoded['proteinDescriptions']]
                    except:
                        data['proteinDescriptions'] = []
                    if aiml.decoded['warnings'] == 0:
                        # data['notes'] += aiml.decoded['summary']
                        pass # il summary è poco informativo
                    else:
                        data['error'] += "Warnings(s): " + " -- ".join([m['message'] for m in aiml.decoded['messages']])
                    mutalyzer_gn_endpoint = "https://mutalyzer.nl/json/getGeneName" #NM_001168465.1
                    '''
                        https://mutalyzer.nl/json/getGeneName?build=hg19&accno=NM_000052.6  ==> {"faultcode": "ENOTFOUND", "faultstring": "Transcript NM_000052.6 not found for build hg19."}
                        https://mutalyzer.nl/json/getGeneName?build=hg19&accno=NM_000052.5  ==> "ATP7A"
                        https://mutalyzer.nl/json/getGeneName?build=hg38&accno=NM_000052.6  ==> "ATP7A"
                        USIAMO LA 38 CHIEDERE CONFERMA A DAVIDE MEI
                    '''
                    url = mutalyzer_gn_endpoint + ("?build=hg38&accno=%s" % aiml.decoded['referenceId'])
                    aign = ApiInvoker(url=url)
                    if aign.decoded:
                        try:
                            gene = HgncGene.objects.get(approved_symbol=aign.decoded)
                            data['gene'] = {
                                'approved_symbol': gene.approved_symbol,
                                'mim_number': gene.mim_number
                            }
                        except:
                            pass
                    # API VEP
                    vep_37_endpoint = "https://grch37.rest.ensembl.org/vep/human/hgvs/"  # NM_001168465.1
                    vep_38_endpoint = "https://rest.ensembl.org/vep/human/hgvs/"  # NM_001168465.1
                    url_37 = vep_37_endpoint + ("%s?content-type=application/json" % request.GET['sv'])
                    url_38 = vep_38_endpoint + ("%s?content-type=application/json" % request.GET['sv'])
                    aivep37 = ApiInvoker(url=url_37)
                    aivep38 = ApiInvoker(url=url_38)
                    type_effect = ''
                    ref = ''
                    ale = ''
                    chromosome_38 = ''
                    position_start_38 = ''
                    position_end_38 = ''
                    chromosome_37 = ''
                    position_start_37 = ''
                    position_end_37 = ''
                    if aivep38.decoded:
                        try:
                            try:
                                type_effect = aivep38.decoded[0]['most_severe_consequence']
                            except:
                                pass
                            try:
                                # "allele_string" quello che è a sinistra dello "/" è ref
                                # "allele_string" quello che è a destra dello "/" è ale
                                allele_string = aivep38.decoded[0]['allele_string']
                                allele_split = allele_string.split('/')
                                ref = allele_split[0]
                                ale = allele_split[1]
                            except:
                                pass
                            try:
                                chromosome_38 = aivep38.decoded[0]['seq_region_name']
                                position_start_38 = aivep38.decoded[0]['start']
                                position_end_38 = aivep38.decoded[0]['end']
                            except:
                                pass
                        except:
                            pass
                    if aivep37.decoded:
                        try:
                            try:
                                chromosome_37 = aivep37.decoded[0]['seq_region_name']
                                position_start_37 = aivep37.decoded[0]['start']
                                position_end_37 = aivep37.decoded[0]['end']
                            except:
                                pass
                        except:
                            pass
                    data['type_effect'] = type_effect
                    data['ref'] = ref
                    data['ale'] = ale
                    data['chromosome_38'] = chromosome_38
                    data['position_start_38'] = position_start_38
                    data['position_end_38'] = position_end_38
                    data['chromosome_37'] = chromosome_37
                    data['position_start_37'] = position_start_37
                    data['position_end_37'] = position_end_37
                else:
                    data['error'] += "Error(s): " + " -- ".join([m['message'] for m in aiml.decoded['messages']])
            else:
                if aics.decoded["messages"][0]['errorcode'] == 'EPARSE':
                    data = aics.decoded
                    data['error'] = " -- ".join([m['message'] for m in aics.decoded['messages']])
                    data['notes'] = ''
                else:
                    try:
                        data['notes'] = " -- ".join([m['message'] for m in aics.decoded['messages']])
                    except:
                        pass
            data['status'] = ApiInvoker.SUCCESS
        else:
            data['error'] = "Generic error."
    except:
        pass
    data['id'] = request.GET['id']
    return JsonResponse(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import forms


class SequenceVariantInput(forms.Widget):
    template_name = 'genetics/widgets/sequence_variant.html'

    def __init__(self, attrs=None):
        super(SequenceVariantInput, self).__init__(attrs)


class SequenceVariantFromGeneInput(forms.Widget):
    template_name = 'genetics/widgets/sequence_variant_from_gene.html'

    def __init__(self, attrs=None):
        super(SequenceVariantFromGeneInput, self).__init__(attrs)


class GeneInput(forms.Widget):
    template_name = 'genetics/widgets/gene.html'

    def __init__(self, attrs=None):
        super(GeneInput, self).__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        """
        Returns this Widget rendered as HTML, as a Unicode string.
        """
        context = self.get_context(name, value, attrs)
        if value is not None:
            from genetics.models import HgncGene
            try:
                context['gene'] = HgncGene.objects.get(pk=value)
            except:
                pass
        return self._render(self.template_name, context, renderer)

